/**
 * Copyright (c) 2009 Jeremy Stribling, Yair Sovran, Irene Zhang, 
 *                    Xavid Pretzer, Jinyang Li, Frans Kaashoek, Robert Morris,
 *     		      Massachusetts Institute of Technology,
 * 		      New York University
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */


#ifndef __WFS_CACHE_BLOCKFILE_H__
#define __WFS_CACHE_BLOCKFILE_H__ 1

#include "wfs_cache.h"
#include <ext/hash_map>

using __gnu_cxx::hash_map;

class wfs_cache_blockfile : public wfs_cache {
  
 protected:
  std::string _cache_dir;
  uint32_t MAX_PERFILE_BUF_SIZE; // 100 M default

 private:

  typedef struct buf_write_entry {
    wfs_buf_write write;
    wfs_version version;
    unsigned long long total_size;
    buf_write_entry *next;
  } buf_write_entry;

  hash_map< wfs_file_id, buf_write_entry* >  _writes;
  hash_map< wfs_file_id, buf_write_entry* >  _last_write;
  hash_map< wfs_file_id, wfs_protocol::fileinfo* >  _md;
  pthread_mutex_t _writes_lock;

  hash_map< wfs_file_id, std::map< std::string, wfs_file_id > > _dirs;
  hash_map< wfs_file_id, std::vector<wfs_storage::dirent> > _readdirs;
  hash_map< wfs_file_id, bool > _complete_dirs;
  hash_map< wfs_file_id, unsigned > _highest_readdir;
  pthread_mutex_t _dirs_lock;
  pthread_mutex_t _readdirs_lock;
  pthread_mutex_t _md_lock;

  std::string filename( wfs_file_handle fh, wfs_version version, 
			unsigned int block_no );
  std::string filename_md( wfs_file_handle fh );

 public:

  wfs_cache_blockfile( std::string cache_dir );
  virtual ~wfs_cache_blockfile();

  int num_buf_writes( wfs_file_handle fh );

  bool read_md( wfs_file_handle fh, 
		wfs_protocol::fileinfo &finfo );
  bool read_any_md( wfs_file_handle fh, wfs_protocol::fileinfo &finfo );
  void store_md( wfs_file_handle fh, 
		 wfs_protocol::fileinfo finfo );
  void writeout_md( wfs_file_handle fh, wfs_version version );
  bool readin_md( wfs_file_handle fh, wfs_protocol::fileinfo &finfo );
  void clear_md( wfs_file_handle fh, wfs_version version );
  
  virtual bool read_block( wfs_file_handle fh, wfs_version version, 
			   unsigned int block_no, std::string &buf );
  virtual bool read_block( wfs_file_handle fh, wfs_version version, 
			   unsigned int block_no );
  virtual void store_block( wfs_file_handle fh, wfs_version version, 
			    unsigned int block_no, std::string buf );
  bool buf_write( wfs_file_handle fh, wfs_version version,
		  unsigned long long offset, bool truncate, 
		  const std::string &buf );
  void get_buf_writes( wfs_file_handle fh, 
		       std::vector<wfs_buf_write> &buf_writes );
  void apply_buf_writes( wfs_file_handle fh, wfs_version version,
			 unsigned int block_no, std::string &buf );

  bool dir_lookup( wfs_file_handle dh, std::string name, wfs_file_handle &fh );
  void add_dir_entry( wfs_file_handle dh, std::string name, 
		      wfs_file_handle fh );
  bool readdir( wfs_file_handle dh, unsigned long long ind, unsigned int n, 
		std::vector<wfs_storage::dirent> &e );
  void add_readdir_entry( wfs_file_handle dh, unsigned long long ind, 
			  std::vector<wfs_storage::dirent> e,
			  bool complete );
  void flush_dir( wfs_file_handle dh );

};

#endif // __WFS_CACHE_BLOCKFILE_H__
