/**
 * Copyright (c) 2009 Jeremy Stribling, Yair Sovran, Irene Zhang, 
 *                    Xavid Pretzer, Jinyang Li, Frans Kaashoek, Robert Morris,
 *     		      Massachusetts Institute of Technology,
 * 		      New York University
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */


#ifndef paxos_protocol_h
#define paxos_protocol_h

#include "rpc.h"

struct prop_t {
  unsigned n;
  std::string m;
};

class paxos_protocol {
 public:
  enum xxstatus { OK, ERR, NOTMASTER, BUSY };
  typedef int status;
  enum rpc_numbers {
    preparereq = 0x11001,
    acceptreq,
    decidereq,
    heartbeat,
  };

  struct preparearg {
    unsigned vid;
    prop_t n;
  };

  struct prepareres {
    int oldview;
    std::vector<std::string> mems;
    int accept;
    prop_t n_a;
    std::vector<std::string> v_a;
    unsigned seqno;
  };

  struct acceptarg {
    unsigned vid;
    unsigned highestseqno;
    std::string highestmem;
    prop_t n;
    std::vector<std::string> v;
  };

  struct decidearg {
    unsigned vid;
    std::vector<std::string> v;
  };

};

inline unmarshall &
operator>>(unmarshall &u, prop_t &a)
{
  u >> a.n;
  u >> a.m;
  return u;
}

inline marshall &
operator<<(marshall &m, prop_t a)
{
  m << a.n;
  m << a.m;
  return m;
}

inline unmarshall &
operator>>(unmarshall &u, paxos_protocol::preparearg &a)
{
  u >> a.vid;
  u >> a.n;
  return u;
}

inline marshall &
operator<<(marshall &m, paxos_protocol::preparearg a)
{
  m << a.vid;
  m << a.n;
  return m;
}

inline unmarshall &
operator>>(unmarshall &u, paxos_protocol::prepareres &r)
{
  u >> r.oldview;
  u >> r.mems;
  u >> r.accept;
  u >> r.n_a;
  u >> r.v_a;
  u >> r.seqno;
  return u;
}

inline marshall &
operator<<(marshall &m, paxos_protocol::prepareres r)
{
  m << r.oldview;
  m << r.mems;
  m << r.accept;
  m << r.n_a;
  m << r.v_a;
  m << r.seqno;
  return m;
}

inline unmarshall &
operator>>(unmarshall &u, paxos_protocol::acceptarg &a)
{
  u >> a.vid;
  u >> a.highestseqno;
  u >> a.highestmem;
  u >> a.n;
  u >> a.v;
  return u;
}

inline marshall &
operator<<(marshall &m, paxos_protocol::acceptarg a)
{
  m << a.vid;
  m << a.highestseqno;
  m << a.highestmem;
  m << a.n;
  m << a.v;
  return m;
}

inline unmarshall &
operator>>(unmarshall &u, paxos_protocol::decidearg &a)
{
  u >> a.vid;
  u >> a.v;
  return u;
}

inline marshall &
operator<<(marshall &m, paxos_protocol::decidearg a)
{
  m << a.vid;
  m << a.v;
  return m;
}

#endif
