/**
 * Copyright (c) 2009 Jeremy Stribling, Yair Sovran, Irene Zhang, 
 *                    Xavid Pretzer, Jinyang Li, Frans Kaashoek, Robert Morris,
 *     		      Massachusetts Institute of Technology,
 * 		      New York University
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */


#ifndef __WFS_CACHE_CLIENT_H__ 
#define __WFS_CACHE_CLIENT_H__ 1

#include <ext/hash_map>
#include <queue>
#include <wfs_lease_manager.h>
#include "wfs_client.h"
#include "wfs_prefetch.h"

using __gnu_cxx::hash_map;

class wfs_prefetch;

class wfs_cache_client : public wfs_client {
 protected:

  wfs_cache *_cache;
  wfs_lease_manager _leases;
  host _host;
  wfs_node _bv_me;
  wfs_fileprefetch _fileprefetch;
  wfs_dirprefetch _dirprefetch;
  bool _read_ahead;
 
  void new_dir_entry( wfs_file_handle dir_handle, 
		      wfs_node dir_node,
		      std::string name, 
		      wfs_file_handle new_fh );

  void lease_renewer();

 public:
  wfs_cache_client( sockaddr_in addr, wfs_node me, slice_manager *slices, 
		    wfs_cache *cache, rpcs *rpc_server, vivaldi *viv, 
		    wfs_client_manager *clients, bool read_ahead, 
		    int prefetch_threads );
  ~wfs_cache_client() { };

  wfs_protocol::status getfile( wfs_file_handle &,
				const std::string &user,
				wfs_protocol::fileinfo &, 
				host *client = NULL );
  wfs_protocol::status getdir( wfs_file_handle &, 
			       const std::string &user, wfs_protocol::dirinfo &,
			       host *client = NULL );
  wfs_protocol::status lookup( wfs_file_handle, std::string name,
			       const std::string &user,  
			       wfs_file_handle &, host *client = NULL );
  virtual wfs_protocol::status readdir( wfs_file_handle, 
					unsigned long long ind, unsigned int n,
					const std::string &user, 
					std::vector<wfs_storage::dirent> &e,
					host *client = NULL );
  virtual wfs_protocol::status readfile( wfs_file_handle, 
					 wfs_version version, 
					 unsigned long long off, 
					 unsigned int n, 
					 const std::string &user, 
					 std::string &buf );
  virtual wfs_protocol::status writefile( wfs_file_handle, wfs_version version, 
					  unsigned long long off, 
					  const std::string &buf,
					  const std::string &user, 
					  wfs_version &new_version,
					  host *client = NULL );
  wfs_protocol::status flush( wfs_file_handle fh, 
			      const std::string &user,
			      wfs_version &new_version,
			      host *client = NULL );
  wfs_protocol::status truncate( wfs_file_handle, wfs_version version,
				 unsigned long long off, 
				 const std::string &user, 
				 wfs_version &new_version,
				 host *client = NULL );
  virtual wfs_protocol::status unlink( wfs_file_handle, 
				       std::string name, 
				       bool is_dir,
				       bool dec_link,
				       const std::string &user, 
				       wfs_protocol::file_and_version &fv,
				       host *client = NULL);
  virtual wfs_protocol::status create( wfs_file_handle, 
				       std::string name, 
				       const std::map<std::string,
				       wfs_permission> &permissions,
				       wfs_file_type type,
				       const std::string &user, 
				       wfs_file_handle &,
				       host *client = NULL);
  virtual wfs_protocol::status rename( wfs_file_handle old_parent, 
				       std::string old_name,
				       wfs_file_handle new_parent, 
				       std::string new_name,
				       const std::string &user, 
				       host *client = NULL);
  virtual wfs_protocol::status chmod( wfs_file_handle fh, 
				      const std::map<std::string,
				      wfs_permission> &permissions, 
				      const std::string &user, 
				      wfs_version &new_version,
				      host *client = NULL);

  virtual wfs_protocol::status link( wfs_file_handle fh, 
				     wfs_file_handle dir_handle, 
				     std::string name, 
				     bool create_file, 
				     bool link_file, 
				     wfs_file_type type,
				     const std::map<std::string,
				     wfs_permission> &permissions,
				     const std::string &user, 
				     wfs_file_handle &new_fh,
				     wfs_version &new_file_version,
				     host *client = NULL );

  wfs_protocol::status invalidate_lease( wfs_file_handle fh, int &dummy );

  virtual wfs_protocol::status read_uncached_data( wfs_file_handle fh, 
						   wfs_version version, 
						   unsigned long long off, 
						   unsigned int n,
						   const std::string &user,
						   std::string &buf );

};

#endif // __WFS_CACHE_CLIENT_H__ 
