/**
 * Copyright (c) 2009 Jeremy Stribling, Yair Sovran, Irene Zhang, 
 *                    Xavid Pretzer, Jinyang Li, Frans Kaashoek, Robert Morris,
 *     		      Massachusetts Institute of Technology,
 * 		      New York University
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */


#ifndef tcpchan_h
#define tcpchan_h

#include "base_chan.h"

// internal. one tcp connection (client or server end).
// multiplexes send()s, un-multiplexes on recv().
class tcpchan : public abstract_channel {
 public:

  tcpchan(int sock, pthread_mutex_t *tcpdelete_m, bool freebuf, 
	  canceller *cancel);
  ~tcpchan();

#ifdef WFS_WITH_LIBSSH
  int connect(const sockaddr_in &dst, ssh_cleanup_frame **frames);
#else
  int connect(const sockaddr_in &dst);
#endif

  int setup(sockaddr_in dst);
  int recv(charbuf &cb);
  bool dead();
  void lose();

  void close();

  const char *getpeername();
  int channo();

  bool verify_user(const char *username);
 private:
  int s;

  ssize_t xwritev(const struct iovec *iovptr, int iovcnt);
#ifdef WFS_WITH_LIBSSH
  int xread(void *p, unsigned int n, ssh_cleanup_frame **frames);
#else
  int xread(void *p, unsigned int n);
#endif

  void do_connect(sockaddr_in dst, short my_port);

};

class tcpbind : public abstract_bind {
private:
  int soc;
  unsigned int port;
public:
  tcpbind(int _soc);
  tcpchan *accept(canceller *cancel,
		  pthread_mutex_t *tcpdelete_m,
		  bool freebuf);
  void close() {::close(soc);}
  unsigned int bound_port() { return port; };
};

class tcphub : public chan_hub {
 public:
  tcpchan *channel(canceller *cancel,
		   pthread_mutex_t *tcpdelete_m,
		   bool freebuf);

  tcpbind *bind(const sockaddr_in &sin);
  int initial_timeout();
};

#endif
