/*
 * Simple program that writes to random places in a file
 * and checks that random reads return the right results.
 */

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>
#include <string.h>
#include <assert.h>
#include <time.h>

int max = 50000;
int flen = 0; // Correct file length.
char *fbytes; // Correct file contents.
char *buf;
int fd;

void rwtest(char *filename);

int
main(int argc, char *argv[])
{
  int seed;
  char *fname;

  if(argc == 1){
    fname = malloc(256);
    sprintf(fname, "rwtest-%d", getpid());
  } else if(argc == 2){
    fname = argv[1];
  } else {
    fprintf(stderr, "Usage: fs-test-4 [filename]\n");
    exit(1);
  }

  seed = time((time_t*) 0);
  seed <<= 16;
  seed |= (getpid() & 0xffff);
  printf("seed 0x%x\n", seed);
  srandom(seed);

  setlinebuf(stdout);

  rwtest(fname);

  printf("fs-test-4: Passed all tests.\n");

  exit(0);
}

void
randomize(char *buf, int count)
{
  int i;

  for(i = 0; i < count; i++) {
    char c;
    // reserve '\0' for the unset bytes
    while( (c = random()) == '\0' );
    buf[i] = c;
  }
}

int
write1(int start, int count)
{
  int cc;

  if(start + count > max)
    count = max - start;
  randomize(buf, count);

  fprintf(stderr, "W %d %d\n", start, count);
 
  if(lseek(fd, start, 0) != start){
    perror("lseek");
    return 0;
  }
  cc = write(fd, buf, count);
  if(cc != count){
    perror("write");
    return 0;
  }

  memcpy(fbytes + start, buf, count);
  if(start + count > flen)
    flen = start + count;

  return 1;
}

int
read1(int start, int count)
{
  int cc, xx, i;
  
  if(start + count > max)
    count = max - start;
  memset(buf, '\0', count);

  fprintf(stderr, "R %d %d\n", start, count);

  if( lseek(fd, start, 0) == -1 ) {
    printf( "Bad seek to %d\n", start );
  }
  cc = read(fd, buf, count);
  xx = flen - start;
  if(xx > count)
    xx = count;
  if(xx < 0)
    xx = 0;
  if(cc != xx){
    fprintf(stderr, "fs-test-4: cc %d xx %d start %d count %d flen %d\n",
            cc, xx, start, count, flen);
    return 0;
  }

  for(i = 0; i < cc; i++){
    if(buf[i] != fbytes[start+i]){
      fprintf(stderr, "fs-test-4: mismatch at %d %02x vs %02x cc %d start %d count %d flen %d\n",
              start+i, buf[i], fbytes[start+i], cc, start, count, flen);
      return 0;
    }
  }

  return 1;
}

void
rwtest(char *filename)
{
  int iters, flags;

  fbytes = malloc(max);
  memset(fbytes, '\0', max);

  buf = malloc(max);

  unlink(filename);
  flags = O_CREAT | O_TRUNC | O_RDWR;
  fd = open(filename, flags, 0666);
  if(fd < 0){
    perror(filename);
    exit(1);
  }

#if 0
  if(!write1(2931, 2069) || !read1(1884, 3116)){
    unlink(filename);
    exit(1);
  }
#endif

  for(iters = 0; iters < 200; iters++){
    if((random() % 2) == 0){
      if(!write1(random() % max, random() % 10000)){
        unlink(filename);
        exit(1);
      }
    }
    if((random() % 2) == 0){
      if(!read1(random() % max, random() % 10000)){
        unlink(filename);
        exit(1);
      }
    }
    if((random() % 31) == 0){
      assert(close(fd) == 0);
      fprintf(stderr, "CO\n");
      fd = open(filename, O_RDWR);
      assert(fd >= 0);
    }
  }

  close(fd);

  unlink(filename);
}
