/*
 * fs-test-8 /classfs/dir1 /classfs/dir2 server1_pid
 *
 * Replication!
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <dirent.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>
#include <signal.h>

char d1[512], d2[512];
extern int errno;
int bigdata_size = 6*1024*1024;

int
fgetlen(int fd)
{
  struct stat sb;

  if(fstat(fd, &sb) < 0){
    fprintf(stderr, "fs-test-8: fstat(): %s\n",
            strerror(errno));
    exit(1);
  }

  return sb.st_size;
}

void
create1(char *d, char *f, char *in)
{
  int fd;
  char n[512];

  sprintf(n, "%s/%s", d, f);
  fd = creat(n, 0666);
  if(fd < 0){
    fprintf(stderr, "fs-test-8: create(%s): %s\n",
            n, strerror(errno));
    exit(1);
  }
  if(fgetlen(fd) != 0){
    fprintf(stderr, "fs-test-8: fstat(%s) size %d after creat\n",
            n, fgetlen(fd));
    exit(1);
  }
  if(write(fd, in, strlen(in)) != strlen(in)){
    fprintf(stderr, "fs-test-8: write(%s): %s\n",
            n, strerror(errno));
    exit(1);
  }
  if(fgetlen(fd) != strlen(in)){
    fprintf(stderr, "fs-test-8: fstat(%s) size %d after wrote %d\n",
            n, fgetlen(fd), (int) strlen(in));
    exit(1);
  }
  if(close(fd) != 0){
    fprintf(stderr, "fs-test-8: close(%s): %s\n",
            n, strerror(errno));
    exit(1);
  }
  usleep(1);
}

void
write1(char *d, char *f, unsigned long long off, char c)
{
  int fd;
  char n[512];

  sprintf(n, "%s/%s", d, f);
  fd = open(n, O_WRONLY);
  if(fd < 0){
    fprintf(stderr, "fs-test-8: write1/open(%s): %s\n",
            n, strerror(errno));
    exit(1);
  }
  if(lseek(fd, off, 0) == -1){
    fprintf(stderr, "fs-test-8: write1/seek(%s): %s\n",
            n, strerror(errno));
    exit(1);
  }
  if(write(fd, &c, 1) != 1){
    fprintf(stderr, "fs-test-8: write1/write(%s): %s\n",
            n, strerror(errno));
    exit(1);
  }
  if(close(fd) != 0){
    fprintf(stderr, "fs-test-8: close(%s): %s\n",
            n, strerror(errno));
    exit(1);
  }
  usleep(1);
}

void
check_correctness(char *n, char *in, int cc, char *buf) {

  if(cc != strlen(in)){
    fprintf(stderr, "fs-test-8: check1 read(%s) returned too little %d vs. %d %s%s\n",
            n,
            cc, strlen(in),
            errno ? ": " : "",
            errno ? strerror(errno) : "");
    exit(1);
  }
  buf[cc] = '\0';
  if(strncmp(buf, in, strlen(in)) != 0){
    fprintf(stderr, "fs-test-8: read(%s) got \"%s\", not \"%s\"\n",
            n, buf, in);
    exit(1);
  }

}

void
check_time(char *n, struct timeval start_tv, struct timeval end_tv, int to_ms ) {

  int total_ms;

  total_ms = (end_tv.tv_sec - start_tv.tv_sec) * 1000 + 
    (float)( (float)(end_tv.tv_usec - start_tv.tv_usec) / 1000);

  if(((float) total_ms) > 10 + ((float) to_ms)) {
    fprintf(stderr, "fs-test-8: check1_maxtime read(%s) took too long at %d ms\n",
            n, total_ms);
    exit(1);
  } else {
    printf( "(to=%d) ", total_ms );
  }

}

void
check1(char *d, char *f, char *in)
{
  int fd, cc;
  char n[512];
  char *buf = (char *) malloc( sizeof(char)*(bigdata_size+1) );

  sprintf(n, "%s/%s", d, f);
  fd = open(n, 0);
  if(fd < 0){
    fprintf(stderr, "fs-test-8: open(%s): %s\n",
            n, strerror(errno));
    exit(1);
  }
  errno = 0;
  cc = read(fd, buf, bigdata_size);

  close(fd);
  check_correctness(n, in, cc, buf);
  free(buf);

}

int does_serve_root(char *d, char *file_not_on_d, pid_t pid, int to_ms) {

  char n[512];
  int fd, answer;

  sprintf(n, "%s/.%d.maxtime/%s", d, to_ms, file_not_on_d);

  kill(pid, SIGSTOP);
  fd = open(n, 0);
  if(fd < 0) {
    answer = 1;
  } else {
    close(fd);
    answer = 0;
  }
  kill(pid, SIGCONT);

  return answer;

}

void
check1_maxtime(char *d, char *f, char *in, int to_ms, pid_t pid, int succeed)
{
  int fd, cc, total_ms;
  char n[512], buf[1000];
  struct timeval start_tv, end_tv;

  sprintf(n, "%s/.%d.maxtime/%s", d, to_ms, f);
  fd = open(n, 0);
  if(fd < 0){
    fprintf(stderr, "fs-test-8: open(%s): %s\n",
            n, strerror(errno));
    exit(1);
  }
  errno = 0;
  gettimeofday(&start_tv, NULL);
  kill(pid, SIGSTOP);
  cc = read(fd, buf, sizeof(buf) - 1);
  gettimeofday(&end_tv, NULL);
  kill(pid, SIGCONT);

  close(fd);

  if(succeed) {
    check_correctness(n, in, cc, buf);
  } else {
    if(cc >= 0){
      fprintf(stderr, "fs-test-8: check1_maxtime read(%s) somehow successful %s%s\n",
	      n,
	      errno ? ": " : "",
	      errno ? strerror(errno) : "");
      exit(1);
    }
  }

  check_time(n, start_tv, end_tv, to_ms);

}

void
check1_killed_during_open(char *d, char *f, char *in, int to_ms, pid_t pid)
{
  int fd, cc, total_ms;
  char n[512], buf[1000];
  struct timeval start_tv, end_tv;

  sprintf(n, "%s/.%d.maxtime/%s", d, to_ms, f);

  gettimeofday(&start_tv, NULL);
  kill(pid, SIGSTOP);
  fd = open(n, 0);
  if(fd < 0){
    fprintf(stderr, "fs-test-8: open(%s): %s\n",
            n, strerror(errno));
    exit(1);
  }
  errno = 0;
  gettimeofday(&end_tv, NULL);
  kill(pid, SIGCONT);


  cc = read(fd, buf, sizeof(buf) - 1);

  close(fd);

  check_time(n, start_tv, end_tv, to_ms);
  check_correctness(n, in, cc, buf);

}


void
unlink1(char *d, char *f)
{
  char n[512];

  sprintf(n, "%s/%s", d, f);
  if(unlink(n) != 0){
    fprintf(stderr, "fs-test-8: unlink(%s): %s\n",
            n, strerror(errno));
    exit(1);
  }
}

void
checknot(char *d, char *f)
{
  int fd;
  char n[512];

  sprintf(n, "%s/%s", d, f);
  fd = open(n, 0);
  if(fd >= 0){
    fprintf(stderr, "fs-test-8: open(%s) succeeded for deleted file\n", n);
    exit(1);
  }
}

int
compar(const void *xa, const void *xb)
{
  char *a = *(char**)xa;
  char *b = *(char**)xb;
  return strcmp(a, b);
}

void
dircheck(char *d, int nf)
{
  DIR *dp;
  struct dirent *e;
  char *names[1000];
  int nnames = 0, i;

  dp = opendir(d);
  if(dp == 0){
    fprintf(stderr, "fs-test-3: opendir(%s): %s\n", d, strerror(errno));
    exit(1);
  }
  while((e = readdir(dp))){
    if(e->d_name[0] != '.'){
      if(nnames >= sizeof(names)/sizeof(names[0])){
        fprintf(stderr, "warning: too many files in %s\n", d);
      }
      names[nnames] = (char *) malloc(strlen(e->d_name) + 1);
      strcpy(names[nnames], e->d_name);
      nnames++;
    }
  }
  closedir(dp);
  
  if(nf != nnames){
    fprintf(stderr, "fs-test-8: wanted %d dir entries, got %d\n", nf, nnames);
    exit(1);
  }

  /* check for duplicate entries */
  qsort(names, nnames, sizeof(names[0]), compar);
  for(i = 0; i < nnames-1; i++){
    if(strcmp(names[i], names[i+1]) == 0){
      fprintf(stderr, "fs-test-8: duplicate directory entry for %s\n", names[i]);
      exit(1);
    }
  }

  for(i = 0; i < nnames; i++)
    free(names[i]);
}

int
main(int argc, char *argv[])
{
  int pid1, pid2, i, bp, bp2;
  char *bd1; 
  char *bd2;
  char *bf;
  char bbv[512];
  char bwo[512];
  char d3[512];
  char d4[512];
  char d5[512];
  char *bigdata = (char *) malloc( sizeof(char)*bigdata_size );

  for( i = 0; i < bigdata_size-1; i++ ) {
    bigdata[i] = (char) ((rand() % 254) + 1);
  }
  bigdata[i] = '\0';

  if(argc != 7){
    fprintf(stderr, "Usage: fs-test-8 dir1 dir2 serv1_pid serv2_pid host1_port host2_port\n");
    exit(1);
  }

  pid1 = atoi(argv[3]);
  pid2 = atoi(argv[4]);

  sprintf(d1, "%s/d%d", argv[1], getpid());
  if(mkdir(d1, 0777) != 0){
    fprintf(stderr, "fs-test-8: failed: mkdir(%s): %s\n",
            d1, strerror(errno));
    exit(1);
  }
  sprintf(d2, "%s/d%d", argv[2], getpid());
  if(access(d2, 0) != 0){
    fprintf(stderr, "fs-test-8: failed: access(%s) after mkdir %s: %s\n",
            d2, d1, strerror(errno));
    exit(1);
  }

  setbuf(stdout, 0);

  // make sure basic replication works by creating a file in a directory, 
  // and then killing that server
  printf( "Replication: " );
  create1( d1, "f1", "aaa" );
  check1( d1, "f1", "aaa" );
  sleep(6);
  kill(pid1, SIGSTOP);
  printf( "(sleeping 6) " );
  sleep(6);
  check1( d2, "f1", "aaa" );
  kill(pid1, SIGCONT);
  printf( "(sleeping 6) " );
  sleep(6);

  unlink1( d2, "f1" );

  printf( "OK\n" );

  // create a file then kill the node before back up can happen
  // create the same file again with different contents on the other node
  // restart the first node and make sure one version is selected
  printf("File conflict merge: ");
  create1(d1, ".1000.lease/.ec/f4", "xxx");
  create1(d1, ".1.synclevel/.1000.lease/.ec/f4", "aaa");
  check1(d1, ".1.synclevel/.1000.lease/.ec/f4", "aaa");
  kill(pid1, SIGSTOP);
  printf("(sleeping 6) ");
  sleep(6);
  create1(d2, ".1.synclevel/.1000.lease/.ec/f4", "bbb");
  check1(d2, ".1.synclevel/.1000.lease/.ec/f4", "bbb");
  kill(pid1, SIGCONT);
  printf("(sleeping 6) ");
  sleep(6);

  DIR *dp;
  struct dirent *e;
  char d_name[50];
  strcpy(d_name,argv[1]);
  strcat(d_name,"/.nodes");
  dp = opendir(d_name);
  int temp[7];
  char rest[50];
  unsigned long long nid, nid1, nid2;
  while ((e = readdir(dp))) {
    sscanf(e->d_name,"%d.%d.%d.%d:%d,%08x%08x",&temp[0],&temp[1],&temp[2],&temp[3],&temp[4],&temp[5],&temp[6]);
    nid = temp[5];
    nid <<= 32;
    nid += temp[6];
    if (temp[4] == atoi(argv[5])) {
      nid1 = nid;
    } else if (temp[4] == atoi(argv[6])){
      nid2 = nid;
    }    
  }
  closedir(dp);
  if (nid1 < nid2) {
    check1(d1, ".1.synclevel/.ec/f4", "bbb");
  } else {
    check1(d2, ".1.synclevel/.ec/f4", "aaa");
  }
  unlink1( d2, "f4" );
  printf(" OK\n");

  // directory merge test
  // create a directory and then create a entry in it.
  // kill the responsible node before backup can happen, and then 
  // wait for the backup to take over.  Make another entry in the directory,
  // and then restart the original resposible node.  Make sure both entries
  // appear.
  printf( "Directory merge: " ); 

  create1(argv[1], "f3a", "aaa");
  create1(argv[2], "f3b", "aaa");
  if(does_serve_root(argv[1], "f3b", pid1, 100)) {
    bd1 = argv[1];
    bd2 = argv[2];
    bp = pid2;
    bp2 = pid1;
  } else {
    bd1 = argv[2];
    bd2 = argv[1];
    bp = pid1;
    bp2 = pid2;
  }

  // create some dummy files so that the RNGs aren't synchronized
  // when creating file ids in a 2-node system
  create1(bd1, "dummy1", "aaa");
  create1(bd1, "dummy2", "aaa");
  unlink1(bd1, "dummy1");
  unlink1(bd1, "dummy2");

  sprintf(d3, "%s/.2.replevel/.ec/.10000.lease/m%d", bd2, getpid());
  if(mkdir(d3, 0777) != 0){
    fprintf(stderr, "fs-test-8: failed: mkdir(%s): %s\n",
            d3, strerror(errno));
    exit(1);
  }

  sprintf(d4, "%s/.2.replevel/.10000.lease/m%d", bd1, getpid());

  create1( d3, "f1", "aaa" ); // synced

  // force no replicas of the directory to be synced.
  kill(bp2, SIGSTOP);
  create1( d3, ".1.synclevel/.1.replevel/f2", "bbb" ); // not synced

  kill(bp, SIGSTOP);
  kill(bp2, SIGCONT);
  printf( "(sleeping 6) " );
  sleep(6);
  // we don't want the lease to extend past the sleeping time, since
  // the maintenance merging code can't invalidate leases ...
  checknot( d4, "f2" );
  create1( d4, ".1.replevel/.1.synclevel/f3", "ccc" ); // synced
  kill(bp, SIGCONT);
  printf( "(sleeping 6) " ); // merging hopefully happens
  sleep(6);
  dircheck( d3, 3 );
  dircheck( d4, 3 );
  check1( d3, "f2", "bbb" );
  check1( d3, "f3", "ccc" );
  check1( d4, "f2", "bbb" );
  check1( d4, "f3", "ccc" );

  unlink1( d3, "f1" );
  unlink1( d3, "f2" );
  unlink1( d3, "f3" );
  unlink1( argv[1], "f3a" );
  unlink1( argv[1], "f3b" );

  printf( "OK\n" );

  printf("fs-test-8: Passed all tests.\n");

  exit(0);
  return(0);
}

