#!/bin/bash -e

d1="$1"
d2="$2"
p1="$3"
p2="$4"

function check1 {
    if cat "$1" > /dev/null 2>&1; then
	:
    else
	echo "File $1 was unreadable when it should have contained '$2'!"
	exit 1
    fi
    if [[ "`cat $1`" != "$2" ]] ; then
	echo "Contents of $1 should be '$2', not '`cat $1`'!"
	exit 1
    fi
}

function checknotreadable {
    # It looks like it exists from the dir entry, so we have to actually
    # try reading it
    if cat "$1" > /dev/null 2>&1 ; then 
	echo "File $1 is readable ('`cat $1`') when it shouldn't be!"
	exit 1
    fi
}

# Test of weird halting bug
# echo -n "Testing weird halting bug..."
# echo -n "abc" > "${d1}/testfile"
# sleep 30
# kill -STOP ${p2}
# echo -n "xyz" >> "${d1}/testfile"
# sleep 10
# kill -CONT ${p2}
# sleep 30
# kill -STOP ${p1}
# check1 "${d2}/testfile" "abcxyz"
# echo "OK."

echo -n "Basic versioning, single server: "
echo -n fish > "${d1}/.versioned/test1"
echo -n dish >> "${d1}/test1"
check1 ${d1}/test1 fishdish
check1 ${d1}/.version=1/test1 ""
check1 ${d1}/.version=2/test1 fish
check1 ${d1}/.version=3/test1 fishdish
checknotreadable ${d1}/.version=4/test1
echo "OK."

echo -n "Basic versioning: "
echo -n fish > "${d1}/.versioned/test2"
echo -n dish >> "${d2}/test2"
check1 ${d1}/test2 fishdish
check1 ${d1}/.version=1/test2 ""
check1 ${d1}/.version=2/test2 fish
check1 ${d1}/.version=3/test2 fishdish
checknotreadable ${d1}/.version=4/test2
check1 ${d2}/test2 fishdish
check1 ${d2}/.version=1/test2 ""
check1 ${d2}/.version=2/test2 fish
check1 ${d2}/.version=3/test2 fishdish
checknotreadable ${d2}/.version=4/test2
echo "OK."

echo -n "Not versioned: "
echo -n "wish" > "${d1}/bone"
echo -n "ing" >> "${d2}/bone"
check1 ${d1}/bone wishing
check1 ${d2}/.version=3/bone wishing
# Version 2 will be in d2's cache, but not d1's
checknotreadable "${d1}/.version=2/bone"
# Version 1 will be in d1's cache, but not d2's
checknotreadable "${d2}/.version=1/bone"
# Version 4 will be in neither cache
checknotreadable "${d1}/.version=4/bone"
checknotreadable "${d2}/.version=4/bone"
echo "OK."

echo -n "With replication: "
mkdir "$d1/repdir"
test -d "$d2/repdir" || (echo "Directory didn't migrate!"; exit 1)
echo -n fish > "${d1}/repdir/.versioned/test4"
echo -n dish > "${d1}/repdir/test4"
sleep 6
kill -s STOP $p1
echo -n "(sleeping 6) "
sleep 6
check1 ${d2}/repdir/test4 dish
check1 ${d2}/repdir/.version=1/test4 ""
check1 ${d2}/repdir/.version=2/test4 fish
check1 ${d2}/repdir/.version=3/test4 ""
check1 ${d2}/repdir/.version=4/test4 dish
checknotreadable ${d2}/repdir/.version=5/test4
kill -s CONT $p1
echo -n "(sleeping 6) "
sleep 6
echo "OK."

echo -n "With replication and lost versions: "
echo -n fish > "${d1}/.versioned/lost4"
sleep 6
kill -s STOP $p2
echo -n dish >> "${d1}/.1.synclevel/lost4"
echo -n tree > "${d1}/.1.synclevel/lost4"
echo -n wombat > "${d1}/.1.synclevel/lost4"
echo -n supercalifragelisticexpialadocious > "${d1}/.1.synclevel/lost4"
sleep 6
kill -s CONT $p2
echo -n "(sleeping 6) "
sleep 6
check1 ${d1}/lost4 supercalifragelisticexpialadocious
check1 ${d1}/.version=1/lost4 ""
check1 ${d1}/.version=2/lost4 fish
check1 ${d1}/.version=3/lost4 fishdish
check1 ${d1}/.version=4/lost4 ""
check1 ${d1}/.version=5/lost4 tree
check1 ${d1}/.version=6/lost4 ""
check1 ${d1}/.version=7/lost4 wombat
check1 ${d1}/.version=8/lost4 ""
checknotreadable ${d1}/.version=10/lost4
kill -s STOP $p1
echo -n "(sleeping 6) "
sleep 6
check1 ${d2}/lost4 supercalifragelisticexpialadocious
check1 ${d2}/.version=1/lost4 ""
check1 ${d2}/.version=2/lost4 fish
check1 ${d2}/.version=3/lost4 fishdish
check1 ${d2}/.version=4/lost4 ""
check1 ${d2}/.version=5/lost4 tree
check1 ${d2}/.version=6/lost4 ""
check1 ${d2}/.version=7/lost4 wombat
check1 ${d2}/.version=8/lost4 ""
checknotreadable ${d2}/.version=10/lost4
kill -s CONT $p1
echo "OK."

echo "fs-test-9: Passed all tests."
