/**
 * Copyright (c) 2009 Jeremy Stribling, Yair Sovran, Irene Zhang, 
 *                    Xavid Pretzer, Jinyang Li, Frans Kaashoek, Robert Morris,
 *     		      Massachusetts Institute of Technology,
 * 		      New York University
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */


#include "wfs_file.h"
#include "wfs_misc.h"
#include <assert.h>
#include <stdlib.h>

// No header file for FUSE client, so hack
#define main ignored_main
#include "wfs_fuse_client.cc"
#undef main

int
main(int argc, char *argv[])
{

  wfs_set_debug(argc-1);

  printf("Testing fill_stat_for_permissions... ");
  std::map<std::string, wfs_permission> perms;
  const char *uname = getenv("USER");
  perms[uname] = WFS_PERMISSION_ADMIN | WFS_PERMISSION_READ | WFS_PERMISSION_WRITE | WFS_PERMISSION_EXECUTE;
  perms[ANYONE_IDENT] = WFS_PERMISSION_READ;
  struct stat st;
  fill_stat_for_permissions(st, perms);
  assert (getuid() == st.st_uid);
  assert (st.st_mode == 0704);
  printf("OK\n");

  printf("Testing permissions_for_mode...");
  wfs_file_handle fh;
  fuse_req_t req = NULL; // Not actually needed if new_file is true...
  perms = permissions_for_mode(0764, getuid(), getgid(), fh,
			       req,true);
  assert (perms.size() == 2);
  assert (perms[getenv("USER")] == (WFS_PERMISSION_ADMIN | WFS_PERMISSION_READ | WFS_PERMISSION_WRITE | WFS_PERMISSION_EXECUTE));
  assert (perms[ANYONE_IDENT] == WFS_PERMISSION_READ);
  printf("OK\n");

  printf("wfs_file-unit-tests: Passed all tests.\n");

  exit(0);
  return(0);
}
