/**
 * Copyright (c) 2009 Jeremy Stribling, Yair Sovran, Irene Zhang, 
 *                    Xavid Pretzer, Jinyang Li, Frans Kaashoek, Robert Morris,
 *     		      Massachusetts Institute of Technology,
 * 		      New York University
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */


#include "ident_util.h"
#include <pwd.h>
#include <stdlib.h>
#include <map>
#include <ext/hash_map>
#include <logging/wfs_log.h>

using __gnu_cxx::hash_map;

const long PW_MAX_NAME = sysconf(_SC_GETPW_R_SIZE_MAX);

static hash_map<uid_t, std::string> pwnam_cache;
std::string get_pwnam_from_uid( uid_t u ) {

  if( pwnam_cache.find(u) != pwnam_cache.end() ) {
    return pwnam_cache[u];
  }

  struct passwd p;
  char name[PW_MAX_NAME];
  struct passwd *res = NULL;
  if (u == (uid_t)-1) // This glitches getpwuid_r
    return "";
  else if( getpwuid_r( u, &p, name, PW_MAX_NAME, &res ) || res == NULL) {
    wfs_log(WFS_DBG_1, "getpwuid failed for user %d\n", u );
    return "";
  }
  std::string s = std::string(p.pw_name);
  pwnam_cache[u] = s;
  return s;
}

std::string current_username() {
  return std::string(getenv("USER"));
}
